#name=M-AUDIO KeyStation Mini 32 MK3
# ↑使用しているデバイスの名前.
import transport # FLのTransport(Play, Stop, Pause & Record)を操作する.
import channels # FLのチャンネルラック操作.
import patterns # FLのパターン操作.
import plugins # FLのプラグイン操作.
import ui # FLのUI操作.

# 定義が見つからないのでここで定義してみる...
SONGLENGTH_MS = 0 # SONGモードの長さをMSで取得.

# MIDI信号/CCの番号.
KS32_VOLUME = 7 # ボリューム
KS32_MOD = 1 # モジュレーション番号.
KS32_SUSTAIN = 64 # サスティン.

VAL_MIN = 0 # 値の最小.
VAL_MAX = 127 # 値は0〜127まで.

## MIDI入力時に呼び出されるコールバック関数.
def OnNoteOn(event):
  # FLが受け取ったMIDI信号を出力.
  print("note on channel:%d number:%d"%(event.status, event.data1))
 
## CCメッセージ受け取り.
def OnControlChange(event):
  # FLが受け取ったCCを出力.
  print("CC channel:%d number:%d"%(event.status, event.data1))
  print("- data2:%d"%event.data2)

  # 選択しているチャンネル番号を取得する.
  channel_idx = channels.channelNumber()
      
  if event.data1 == KS32_VOLUME:
    # ノブの位置で再生位置を移動してみる.
    if transport.getLoopMode() == 0:
      # パターンモード.
      idx = patterns.patternNumber() # 現在選択しているパターン.
      total = patterns.getPatternLength(idx) # パターンの長さ.
      pos = int(total * event.data2 / VAL_MAX)
      print("pos:", pos , " total:", total)
      transport.setSongPos(1.0 * pos / total)
    else:
      # ソングモード.
      total = transport.getSongLength(SONGLENGTH_MS) # 曲の長さ.
      pos = total * event.data2 / VAL_MAX
      print("pos:", pos , " total:", total)
      transport.setSongPos(pos, SONGLENGTH_MS)
    event.handled = True # FLに処理させる.
    
  elif event.data1 == KS32_MOD:
    # ループモード切り替え.
    if event.data2 == 127: # 入力時のみ.
      transport.setLoopMode()
      event.handled = True # FLに処理させる
  elif event.data1 == KS32_SUSTAIN:
    # 再生・停止
    if transport.isPlaying(): # 再生中
      transport.stop() # 停止.
    else: # 停止している.
      transport.start() # 再生.
    event.handled = True # FLに処理させる
 
## プログラムチェンジメッセージのコールバック関数.
def OnProgramChange(event):
  # FLが受け取ったMIDI信号を出力.
  print("program change channel:%d number:%d"%(event.status, event.data1))
 
## システムエクスクルーシブのコールバック関数.
def OnSysEx(event):
    # FLが受け取ったMIDI信号を出力.
    print("sys ex channel:%d number:%d"%(event.status, event.data1))
 
## ピッチベンドのコールバック関数.
def OnPitchBend(event):
  # イベントを処理する場合はこれをTrueにする.
  event.handled = False
 
  # FLが受け取ったMIDI信号を出力.
  print("pitch bend channel:%d number:%d"%(event.status, event.data1))
  print("- data2:%d"%event.data2)
   
  if event.data2 == 127:
    # ピッチを上げる.
    # 選択しているチャンネル番号を返す.
    idx = channels.selectedChannel()
    channels.selectOneChannel(idx+1) # 1つ上を選択する.
    print("selectOneChannel(%d)"%idx)
    event.handled = True # FLに処理させる.
  elif event.data2 == 0:
    # ピッチを下げる.
    print("setFocused(widChannelRack)")
    # 選択しているチャンネル番号を返す.
    idx = channels.selectedChannel()
    channels.selectOneChannel(idx-1) # 1つ下を選択する.
    print("selectOneChannel(%d)"%idx)
    event.handled = True # FLに処理させる.
 